@@ -3,7 +3,7 @@
  * @brief Member definitions for the ArpWidget GUI class.
  *
  *
- *      Copyright 2009 - 2017 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009 - 2024 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -26,8 +26,10 @@
 
 #include <QLineEdit>
 
+#ifdef APPBUILD
 #include "midiarp.h"
-#include "inoutbox.h"
+#endif
+#include "modulewidget.h"
 #include "slider.h"
 #include "arpscreen.h"
 
@@ -40,11 +42,13 @@
  * and interacts with it.
 class ArpWidget : public QWidget
 */
-class ArpWidget :  public InOutBox
+class ArpWidget :  public ModuleWidget
 {
   Q_OBJECT
 
+#ifdef APPBUILD
     MidiArp *midiArp;
+#endif
 
     QGroupBox *randomBox, *envelopeBox;
     QToolButton *textEditButton, *textStoreButton, *textRemoveButton;
@@ -65,18 +69,15 @@ class ArpWidget :  public InOutBox
  * @param portCount Number of available MIDI output ports
  * @param compactStyle If set to True, Widget will use reduced spacing and small fonts
  * @param mutedAdd If set to True, the module will be added in muted state
- * @param inOutVisible Add the module with visible InOutBox or not
+ * @param inOutVisible Add the module with visible ModuleWidget or not
  * @param name The name of the module preceded by its type (Arp: , etc...)
  */
-    ArpWidget(MidiArp *p_midiArp, GlobStore *p_globStore,
-            int portCount, bool compactStyle,
-            bool mutedAdd = false, bool inOutVisible = true,
+    ArpWidget(MidiArp *p_midiArp, GlobStore *p_globStore, Prefs *p_prefs,
+            bool inOutVisible = true,
             const QString& name = "");
 
 #else
-    ArpWidget(
-            bool compactStyle,
-            bool mutedAdd = false, bool inOutVisible = true);
+    ArpWidget();
 #endif
 
     ArpScreen *screen;
@@ -93,7 +94,13 @@ class ArpWidget :  public InOutBox
     QStringList patternPresets, patternNames;
     QAction *latchModeAction;
 
-
+/*! 
+ * @brief ENUM for Internal MIDI Control IDs supported
+ * by arp widget 
+ */
+    enum ARPWIDGET_CTRL_IDS {
+        ARP_PRESET_SWITCH = 1,
+    };
 
 #ifdef APPBUILD
     //these members are common to all modules
@@ -102,15 +109,16 @@ class ArpWidget :  public InOutBox
 * passed by the caller, i.e. MainWindow.
 *
 * @param xml QXmlStreamWriter to read from
+* @param qmaxVersion QString with xml format version
 */
-    void readData(QXmlStreamReader& xml);
+    void readData(QXmlStreamReader& xml, const QString& qmaxVersion) override;
 /*!
 * @brief writes all parameters of this module to an XML stream
 * passed by the caller, i.e. MainWindow.
 *
 * @param xml QXmlStreamWriter to write to
 */
-    void writeData(QXmlStreamWriter& xml);
+    void writeData(QXmlStreamWriter& xml) override;
 /*!
 * @brief returns the MidiArp instance associated with this GUI
 * Widget.
@@ -122,9 +130,9 @@ class ArpWidget :  public InOutBox
     void doRestoreParams(int ix);
     void updateDisplay();
     void handleController(int ccnumber, int channel, int value);
-#endif
 
     void updateCursorPos() { screen->updateCursor(midiArp->getFramePtr()); }
+#endif
     
 /* SIGNALS */
   signals:
