@@ -3,7 +3,7 @@
  * @brief Implements the the LV2 GUI for the QMidiArp Arp plugin.
  *
  *
- *      Copyright 2009 - 2017 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009 - 2024 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@ ArpWidgetLV2::ArpWidgetLV2 (
         LV2UI_Controller ct,
         LV2UI_Write_Function write_function,
         const LV2_Feature *const *host_features)
-        : ArpWidget(true, true, true)
+        : ArpWidget()
 {
     m_controller = ct;
     writeFunction = write_function;
@@ -41,10 +41,6 @@ ArpWidgetLV2::ArpWidgetLV2 (
     for (int i = 0; host_features[i]; ++i) {
         if (::strcmp(host_features[i]->URI, LV2_URID_URI "#map") == 0) {
             urid_map = (LV2_URID_Map *) host_features[i]->data;
-            if (urid_map) {
-                (void)urid_map->map(urid_map->handle, LV2_MIDI_EVENT_URI);
-                break;
-            }
         }
     }
     if (!urid_map) {
@@ -114,6 +110,7 @@ ArpWidgetLV2::ArpWidgetLV2 (
     mouseXCur = 0.0;
     mouseYCur = 0.0;
     receivedPatternOnce = false;
+    uiIsUp = true;
     sendUIisUp(true);
 }
 
@@ -162,8 +159,10 @@ void ArpWidgetLV2::port_event ( uint32_t port_index,
                     chIn->setCurrentIndex(fValue);
             break;
             case CURSOR_POS:
-                    screen->updateCursor((int)fValue);
-                    screen->update();
+                    if (screen->currentIndex != (int)fValue) {
+                        screen->updateCursor((int)fValue);
+                        screen->update();
+                    }
             break;
             case PATTERN_PRESET:
                     //patternPresetBox->setCurrentIndex(fValue);
@@ -302,9 +301,10 @@ void ArpWidgetLV2::updatePattern(const QString& p_pattern)
                 break;
 
             case 'p':
-                if (!chordmd)
+                if (!chordmd) {
                     nsteps += stepwd;
                     npoints++;
+                }
                 break;
 
             case '+':
@@ -362,6 +362,7 @@ void ArpWidgetLV2::sendUIisUp(bool on)
     uint8_t obj_buf[64];
     int state;
 
+    uiIsUp = on;
     LV2_Atom_Forge_Frame frame;
     lv2_atom_forge_frame_time(&forge, 0);
 
@@ -408,8 +409,9 @@ void ArpWidgetLV2::mapBool(bool on)
 {
     float value = (float)on;
     if (muteOutAction == sender()) {
-        updateParam(MUTE, value);
         screen->setMuted(value);
+        screen->update();
+        updateParam(MUTE, value);
     }
     else if (deferChangesAction == sender())    updateParam(DEFER, value);
     else if (latchModeAction == sender())       updateParam(LATCH_MODE, value);
@@ -445,17 +447,176 @@ void ArpWidgetLV2::updateParam(int index, float fValue) const
         writeFunction(m_controller, index, sizeof(float), 0, &fValue);
 }
 
+bool ArpWidgetLV2::isIdleClosed()
+{
+    return (uiIsUp == false);
+}
+
+
+//=== The following comes from the synthv1 plugin by rncbc ====
+QApplication *ArpWidgetLV2::g_qAppInstance = nullptr;
+unsigned int  ArpWidgetLV2::qAppCount = 0;
+
+
+void ArpWidgetLV2::qAppInstantiate(void)
+{
+    if (qApp == nullptr && g_qAppInstance == nullptr) {
+        static int s_argc = 1;
+        static const char *s_argv[] = { __func__, nullptr };
+        g_qAppInstance = new QApplication(s_argc, (char **) s_argv);
+    }
+
+    if (g_qAppInstance)
+        qAppCount++;
+}
+
+void ArpWidgetLV2::qAppCleanup (void)
+{
+    if (g_qAppInstance && --qAppCount == 0) {
+        delete g_qAppInstance;
+        g_qAppInstance = nullptr;
+    }
+}
+
+QApplication *ArpWidgetLV2::qAppInstance(void)
+{
+    return g_qAppInstance;
+}
+/*
+// Now deprecated LV2 feature
+int MidiArpLV2ui_resize ( LV2UI_Handle ui, int width, int height )
+{
+    ArpWidgetLV2 *pWidget = static_cast<ArpWidgetLV2 *> (ui);
+    if (pWidget) {
+        pWidget->resize(width, height);
+        return 0;
+    } else {
+        return 1;
+    }
+}
+*/
+
+int MidiArpLV2ui_idle ( LV2UI_Handle ui )
+{
+    ArpWidgetLV2 *pWidget = static_cast<ArpWidgetLV2 *> (ui);
+    if  (pWidget && !pWidget->isIdleClosed()) {
+        QApplication::processEvents();
+        return 0;
+    } else {
+        return 1;
+    }
+}
+
+int MidiArpLV2ui_show ( LV2UI_Handle ui )
+{
+    ArpWidgetLV2 *pWidget = static_cast<ArpWidgetLV2 *> (ui);
+    if (pWidget) {
+        pWidget->show();
+        pWidget->raise();
+        pWidget->activateWindow();
+        return 0;
+    } else {
+        return 1;
+    }
+}
+
+int MidiArpLV2ui_hide ( LV2UI_Handle ui )
+{
+    ArpWidgetLV2 *pWidget = static_cast<ArpWidgetLV2 *> (ui);
+    if (pWidget) {
+        pWidget->hide();
+        return 0;
+    } else {
+        return 1;
+    }
+}
+/*
+static const LV2UI_Resize MidiArpLV2ui_resize_interface =
+{
+    nullptr, // handle: host should use its own when calling ui_resize().
+    MidiArpLV2ui_resize
+};
+*/
+static const LV2UI_Idle_Interface MidiArpLV2ui_idle_interface =
+{
+    MidiArpLV2ui_idle
+};
+
+static const LV2UI_Show_Interface MidiArpLV2ui_show_interface =
+{
+    MidiArpLV2ui_show,
+    MidiArpLV2ui_hide
+};
+
+
+static const void *MidiArpLV2ui_extension_data ( const char *uri )
+{
+    /*
+    if (::strcmp(uri, LV2_UI__resize) == 0)
+        return (void *) &MidiArpLV2ui_resize_interface;
+    else 
+    */
+    if (::strcmp(uri, LV2_UI__idleInterface) == 0)
+        return (void *) &MidiArpLV2ui_idle_interface;
+    else 
+    if (::strcmp(uri, LV2_UI__showInterface) == 0)
+        return (void *) &MidiArpLV2ui_show_interface;
+     else
+        return nullptr;
+}
+
+//===
+/*
 static LV2UI_Handle MidiArpLV2ui_instantiate (
     const LV2UI_Descriptor *, const char *, const char *,
     LV2UI_Write_Function write_function,
     LV2UI_Controller controller, LV2UI_Widget *widget,
     const LV2_Feature *const *host_features )
 {
+    ArpWidgetLV2::qAppInstantiate();
     ArpWidgetLV2 *pWidget = new ArpWidgetLV2(
                     controller, write_function, host_features);
     *widget = pWidget;
     return pWidget;
 }
+*/
+static LV2UI_Handle MidiArpLV2ui_x11_instantiate (
+    const LV2UI_Descriptor *, const char *, const char *,
+    LV2UI_Write_Function write_function,
+    LV2UI_Controller controller, LV2UI_Widget *widget,
+    const LV2_Feature *const *ui_features )
+{
+    WId winid, parent = 0;
+    //LV2UI_Resize *resize = nullptr;
+
+    for (int i = 0; ui_features[i]; ++i) {
+        if (::strcmp(ui_features[i]->URI, LV2_UI__parent) == 0)
+            parent = (WId) ui_features[i]->data;
+        /*
+        else
+        if (::strcmp(ui_features[i]->URI, LV2_UI__resize) == 0)
+            resize = (LV2UI_Resize *) ui_features[i]->data;
+        */
+    }
+
+    if (!parent)
+        return nullptr;
+
+    ArpWidgetLV2::qAppInstantiate();
+    ArpWidgetLV2 *pWidget
+        = new ArpWidgetLV2(controller, write_function, ui_features);
+    /*
+    if (resize && resize->handle) {
+        const QSize& hint = pWidget->sizeHint();
+        resize->ui_resize(resize->handle, hint.width(), hint.height());
+    }
+    */
+    winid = pWidget->winId();
+    pWidget->windowHandle()->setParent(QWindow::fromWinId(parent));
+    pWidget->show();
+    *widget = (LV2UI_Widget) winid;
+    return pWidget;
+}
 
 static void MidiArpLV2ui_cleanup ( LV2UI_Handle ui )
 {
@@ -464,6 +625,7 @@ static void MidiArpLV2ui_cleanup ( LV2UI_Handle ui )
         pWidget->sendUIisUp(false);
         delete pWidget;
     }
+    ArpWidgetLV2::qAppCleanup();
 }
 
 static void MidiArpLV2ui_port_event (
@@ -474,12 +636,7 @@ static void MidiArpLV2ui_port_event (
     if (pWidget)
         pWidget->port_event(port_index, buffer_size, format, buffer);
 }
-
-static const void *MidiArpLV2ui_extension_data ( const char * )
-{
-    return NULL;
-}
-
+/*
 static const LV2UI_Descriptor MidiArpLV2ui_descriptor =
 {
     QMIDIARP_ARP_LV2UI_URI,
@@ -488,9 +645,25 @@ static const LV2UI_Descriptor MidiArpLV2ui_descriptor =
     MidiArpLV2ui_port_event,
     MidiArpLV2ui_extension_data
 };
+*/
+static const LV2UI_Descriptor MidiArpLV2ui_x11_descriptor =
+{
+    QMIDIARP_ARP_LV2UI_X11_URI,
+    MidiArpLV2ui_x11_instantiate,
+    MidiArpLV2ui_cleanup,
+    MidiArpLV2ui_port_event,
+    MidiArpLV2ui_extension_data
+};
 
 LV2_SYMBOL_EXPORT const LV2UI_Descriptor *lv2ui_descriptor ( uint32_t index )
 {
-    return (index == 0 ? &MidiArpLV2ui_descriptor : NULL);
+    if (index == 0)
+        return &MidiArpLV2ui_x11_descriptor;
+    /*
+    else
+    if (index == 1)
+        return &MidiArpLV2ui_descriptor;
+    */
+    else
+        return NULL;
 }
-
