@@ -3,7 +3,7 @@
  * @brief Implements the SeqWidget GUI class.
  *
  *
- *      Copyright 2009 - 2017 <qmidiarp-devel@lists.sourceforge.net>
+ *      Copyright 2009 - 2024 <qmidiarp-devel@lists.sourceforge.net>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -30,18 +30,16 @@
 
 #ifdef APPBUILD
 SeqWidget::SeqWidget(MidiSeq *p_midiSeq, GlobStore *p_globStore,
-    int portCount, bool compactStyle,
-    bool mutedAdd, bool inOutVisible, const QString& p_name):
-    InOutBox(p_midiSeq, p_globStore, portCount, compactStyle, inOutVisible, p_name),
+    Prefs *p_prefs, bool inOutVisible, const QString& p_name):
+    ModuleWidget(p_midiSeq, p_globStore, p_prefs, inOutVisible, p_name),
     midiSeq(p_midiSeq)
 {
+    bool compactStyle = p_prefs->compactStyle;
 #else
-SeqWidget::SeqWidget(
-    bool compactStyle,
-    bool mutedAdd, bool inOutVisible):
-    InOutBox(compactStyle, inOutVisible, "Seq:"),
-    midiSeq(NULL)
+SeqWidget::SeqWidget():
+    ModuleWidget("Seq:")
 {
+    bool compactStyle = true;
 #endif
 
     // group box for sequence setup
@@ -67,7 +65,7 @@ SeqWidget::SeqWidget(
     connect(loopBox, SIGNAL(activated(int)), this,
             SLOT(updateLoop(int)));
 #ifdef APPBUILD
-    midiControl->addMidiLearnMenu("LoopMode", loopBox, 6);
+    midiControl->addMidiLearnMenu("LoopMode", loopBox, SEQ_LOOP_MODE);
 #endif
     QLabel *recordButtonLabel = new QLabel(tr("Re&cord"));
     recordAction = new QAction(QPixmap(seqrecord_xpm), tr("Re&cord"), this);
@@ -78,40 +76,44 @@ SeqWidget::SeqWidget(
     recordButtonLabel->setBuddy(recordButton);
     connect(recordAction, SIGNAL(toggled(bool)), this, SLOT(setRecord(bool)));
 #ifdef APPBUILD
-    midiControl->addMidiLearnMenu("RecordToggle", recordButton, 3);
+    midiControl->addMidiLearnMenu("RecordToggle", recordButton, SEQ_RECORD);
 #endif
 
     QLabel *resBoxLabel = new QLabel(tr("&Resolution"));
     resBox = new QComboBox;
     resBoxLabel->setBuddy(resBox);
     names.clear();
-    names << "1" << "2" << "4" << "8" << "16";
+    for (uint64_t i = 0; i < sizeof(seqResValues)/sizeof(seqResValues[0]); i++) {
+        names << QString::number(seqResValues[i]);
+    }
     resBox->insertItems(0, names);
-    resBox->setCurrentIndex(2);
-    resBoxIndex = 2;
+    resBoxIndex = 3;
+    resBox->setCurrentIndex(resBoxIndex);
     resBox->setToolTip(
             tr("Resolution (notes/beat): Number of notes produced every beat"));
     resBox->setMinimumContentsLength(3);
     connect(resBox, SIGNAL(activated(int)), this,
             SLOT(updateRes(int)));
 #ifdef APPBUILD
-    midiControl->addMidiLearnMenu("Resolution", resBox, 4);
+    midiControl->addMidiLearnMenu("Resolution", resBox, SEQ_RESOLUTION);
 #endif
 
     QLabel *sizeBoxLabel = new QLabel(tr("&Length"));
     sizeBox = new QComboBox;
     sizeBoxLabel->setBuddy(sizeBox);
     names.clear();
-    names << "1" << "2" << "3" << "4" << "5" << "6" << "7" << "8" << "16" << "32";
+    for (uint64_t i = 0; i < sizeof(seqSizeValues)/sizeof(seqSizeValues[0]); i++) {
+        names << QString::number(seqSizeValues[i]);
+    }
     sizeBox->insertItems(0, names);
-    sizeBox->setCurrentIndex(3);
     sizeBoxIndex = 3;
+    sizeBox->setCurrentIndex(sizeBoxIndex);
     sizeBox->setToolTip(tr("Length of Sequence in beats"));
     sizeBox->setMinimumContentsLength(3);
     connect(sizeBox, SIGNAL(activated(int)), this,
             SLOT(updateSize(int)));
 #ifdef APPBUILD
-    midiControl->addMidiLearnMenu("Size", sizeBox, 5);
+    midiControl->addMidiLearnMenu("Size", sizeBox, SEQ_SIZE);
 #endif
 
     dispSignalMapper = new QSignalMapper(this);
@@ -146,7 +148,7 @@ SeqWidget::SeqWidget(
     connect(velocity, SIGNAL(valueChanged(int)), this,
             SLOT(updateVelocity(int)));
 #ifdef APPBUILD
-    midiControl->addMidiLearnMenu("Velocity", velocity, 1);
+    midiControl->addMidiLearnMenu("Velocity", velocity, SEQ_VELOCITY);
 #endif
 
     notelength = new Slider(0, 127, 1, 16, 60, Qt::Horizontal,
@@ -154,15 +156,15 @@ SeqWidget::SeqWidget(
     connect(notelength, SIGNAL(valueChanged(int)), this,
             SLOT(updateNoteLength(int)));
 #ifdef APPBUILD
-    midiControl->addMidiLearnMenu("NoteLength", notelength, 2);
+    midiControl->addMidiLearnMenu("NoteLength", notelength, SEQ_NOTE_LENGTH);
 #endif
 
-    transpose = new Slider(-24, 24, 1, 2, 0, Qt::Horizontal,
+    transpose = new Slider(-36, 24, 1, 2, 0, Qt::Horizontal,
             tr("&Transpose"), this);
     connect(transpose, SIGNAL(valueChanged(int)), this,
             SLOT(updateTranspose(int)));
 #ifdef APPBUILD
-    midiControl->addMidiLearnMenu("Transpose", transpose, 8);
+    midiControl->addMidiLearnMenu("Transpose", transpose, SEQ_TRANSPOSE);
 #endif
 
     QGridLayout* sliderLayout = new QGridLayout;
@@ -204,9 +206,8 @@ SeqWidget::SeqWidget(
     widgetLayout->addWidget(hideInOutBoxButton, 0);
     widgetLayout->addWidget(inOutBoxWidget, 0);
 
-    muteOutAction->setChecked(mutedAdd);
 #ifdef APPBUILD
-    midiControl->addMidiLearnMenu("Out Channel", channelOut, 9);
+    midiControl->addMidiLearnMenu("Out Channel", channelOut, SEQ_CHANNEL_OUT);
 #endif
 
     setLayout(widgetLayout);
@@ -263,7 +264,7 @@ void SeqWidget::writeData(QXmlStreamWriter& xml)
         tempArray.clear();
         l1 = 0;
         while (l1 < midiSeq->maxNPoints) {
-            tempArray.append(midiSeq->customWave.at(l1).value);
+            tempArray.append(midiSeq->customWave.at(l1).data);
             l1++;
         }
         xml.writeStartElement("sequence");
@@ -275,10 +276,10 @@ void SeqWidget::writeData(QXmlStreamWriter& xml)
     xml.writeEndElement();
 }
 
-void SeqWidget::readData(QXmlStreamReader& xml)
+void SeqWidget::readData(QXmlStreamReader& xml, const QString& qmaxVersion)
 {
     int tmp;
-    Sample sample;
+    Sample sample = {0, 0, 0, false};
 
     while (!xml.atEnd()) {
         xml.readNext();
@@ -286,6 +287,16 @@ void SeqWidget::readData(QXmlStreamReader& xml)
             break;
             
         readCommonData(xml);
+            
+        // Compatibility with earlier versions //
+        for (int l1 = 0; l1 < parStore->list.count(); l1++) {
+            if (qmaxVersion == "" && parStore->list[l1].res < 5) {
+                parStore->list[l1].res = mapOldSeqRes[parStore->list[l1].res];
+            }
+            if (qmaxVersion == "" && parStore->list[l1].size < 10) {
+                parStore->list[l1].size = mapOldSeqSize[parStore->list[l1].size];
+            }
+        }
         
         if (xml.isStartElement() && (xml.name() == "display")) {
             while (!xml.atEnd()) {
@@ -310,11 +321,17 @@ void SeqWidget::readData(QXmlStreamReader& xml)
                 }
                 else if (xml.name() == "resolution") {
                     tmp = xml.readElementText().toInt();
+                    if (qmaxVersion == "" && tmp < 5) {
+                        tmp = mapOldSeqRes[tmp];
+                    }
                     resBox->setCurrentIndex(tmp);
                     updateRes(tmp);
                 }
                 else if (xml.name() == "size") {
                     tmp = xml.readElementText().toInt();
+                    if (qmaxVersion == "" && tmp < 10) {
+                        tmp = mapOldSeqSize[tmp];
+                    }
                     sizeBox->setCurrentIndex(tmp);
                     updateSize(tmp);
                 }
@@ -361,7 +378,7 @@ void SeqWidget::readData(QXmlStreamReader& xml)
                     int step = TPQN / midiSeq->res;
                     int lt = 0;
                     for (int l1 = 0; l1 < tmpArray.count(); l1++) {
-                        sample.value = tmpArray.at(l1);
+                        sample.data = tmpArray.at(l1);
                         sample.tick = lt;
                         sample.muted = midiSeq->muteMask[l1];
                         midiSeq->customWave[l1] = sample;
@@ -390,7 +407,11 @@ void SeqWidget::readData(QXmlStreamReader& xml)
 
 void SeqWidget::updateNoteLength(int val)
 {
-    if (midiSeq) midiSeq->updateNoteLength(sliderToTickLen(val));
+#ifdef APPBUILD
+    midiSeq->updateNoteLength(sliderToTickLen(val));
+#else
+    (void)val;
+#endif
     modified = true;
 }
 
@@ -398,80 +419,101 @@ void SeqWidget::updateWaveForm(int val)
 {
     (void)val;
     modified = true;
-    if (!midiSeq) return;
+#ifdef APPBUILD
     std::vector<Sample> sdata;
     midiSeq->getData(&sdata);
+#if QT_VERSION <= QT_VERSION_CHECK(5,14,0)
     data=QVector<Sample>::fromStdVector(sdata);
+#else
+    data=QVector<Sample>(sdata.begin(), sdata.end());
+#endif
     screen->updateData(data);
+#endif
 }
 
 void SeqWidget::setRecord(bool on)
 {
     recordMode = on;
     screen->setRecordMode(on);
-    if (!midiSeq) return;
+#ifdef APPBUILD
     midiSeq->setRecordMode(on);
     screen->setCurrentRecStep(midiSeq->currentRecStep);
+#endif
 }
 
 void SeqWidget::updateRes(int val)
 {
-    if (val > 4) return;
+    if ((uint64_t)val >= sizeof(seqResValues)/sizeof(seqResValues[0])) return;
     resBoxIndex = val;
     modified = true;
-    if (!midiSeq) return;
+#ifdef APPBUILD
     midiSeq->res = seqResValues[val];
     midiSeq->resizeAll();
     std::vector<Sample> sdata;
     midiSeq->getData(&sdata);
+#if QT_VERSION <= QT_VERSION_CHECK(5,14,0)
     data=QVector<Sample>::fromStdVector(sdata);
+#else
+    data=QVector<Sample>(sdata.begin(), sdata.end());
+#endif
     screen->setCurrentRecStep(midiSeq->currentRecStep);
     screen->updateData(data);
+#endif
 }
 
 void SeqWidget::updateSize(int val)
 {
-    if (val > 9) return;
+    if ((uint64_t)val >= sizeof(seqSizeValues)/sizeof(seqSizeValues[0])) return;
     sizeBoxIndex = val;
     modified = true;
-    if (!midiSeq) return;
+#ifdef APPBUILD
     midiSeq->size = sizeBox->currentText().toInt();
     midiSeq->resizeAll();
     std::vector<Sample> sdata;
     midiSeq->getData(&sdata);
+#if QT_VERSION <= QT_VERSION_CHECK(5,14,0)
     data=QVector<Sample>::fromStdVector(sdata);
+#else
+    data=QVector<Sample>(sdata.begin(), sdata.end());
+#endif
     screen->setCurrentRecStep(midiSeq->currentRecStep);
     screen->updateData(data);
+#endif
 }
 
 void SeqWidget::updateLoop(int val)
 {
     if (val > 6) return;
-    if (midiSeq) midiSeq->updateLoop(val);
+#ifdef APPBUILD
+    midiSeq->updateLoop(val);
+#else
+    (void)val;
+#endif
     modified = true;
 }
 
 void SeqWidget::updateVelocity(int val)
 {
-    if (midiSeq) midiSeq->updateVelocity(val);
+#ifdef APPBUILD
+    midiSeq->updateVelocity(val);
+#else
+    (void)val;
+#endif
     modified = true;
 }
 
 void SeqWidget::updateTranspose(int val)
 {
-    if (midiSeq) midiSeq->updateTranspose(val);
+#ifdef APPBUILD
+    midiSeq->updateTranspose(val);
+#else
+    (void)val;
+#endif
     modified = true;
 }
 
 void SeqWidget::mouseEvent(double mouseX, double mouseY, int buttons, int pressed)
 {
-    if (!midiSeq) {
-        emit mouseSig(mouseX, mouseY, buttons, pressed);
-    }
-    else {
-        midiSeq->mouseEvent(mouseX, mouseY, buttons, pressed);
-    }
-
     if ((mouseY < 0) && (pressed != 2)) { // we have to recalculate loopMarker for screen update
         if (mouseX < 0) mouseX = 0;
         if (buttons == 2) mouseX = - mouseX;
@@ -483,6 +525,12 @@ void SeqWidget::mouseEvent(double mouseX, double mouseY, int buttons, int presse
         screen->setLoopMarker(lm);
         screen->updateDraw();
     }
+#ifdef APPBUILD
+    midiSeq->mouseEvent(mouseX, mouseY, buttons, pressed);
+#else
+    emit mouseSig(mouseX, mouseY, buttons, pressed);
+#endif
+
     modified = true;
 }
 
@@ -494,7 +542,9 @@ void SeqWidget::setDispVert(int mode)
 void SeqWidget::updateDispVert(int mode)
 {
     dispVertIndex = mode;
-    if (midiSeq) midiSeq->updateDispVert(mode);
+#ifdef APPBUILD
+    midiSeq->updateDispVert(mode);
+#endif
     screen->updateDispVert(mode);
     modified = true;
 }
@@ -546,12 +596,14 @@ void SeqWidget::doRestoreParams(int ix)
     updateLoop(parStore->list.at(ix).loopMode);
     updateWaveForm(parStore->list.at(ix).waveForm);
     midiSeq->setFramePtr(0);
-
+    
     needsGUIUpdate = true;
 }
 
-void SeqWidget::copyParamsFrom(SeqWidget *fromWidget)
+void SeqWidget::copyParamsFrom(ModuleWidget *p_fromWidget)
 {
+    SeqWidget *fromWidget = (SeqWidget *)p_fromWidget;
+    
     int tmp;
     setDispVert(fromWidget->dispVertIndex);
     enableNoteIn->setChecked(fromWidget->enableNoteIn->isChecked());
@@ -611,12 +663,20 @@ void SeqWidget::copyParamsFrom(SeqWidget *fromWidget)
 
 QVector<Sample> SeqWidget::getCustomWave()
 {
+#if QT_VERSION <= QT_VERSION_CHECK(5,14,0)
     return QVector<Sample>::fromStdVector(midiSeq->customWave);
+#else
+    return QVector<Sample>(midiSeq->customWave.begin(), midiSeq->customWave.end());
+#endif
 }
 
 QVector<bool> SeqWidget::getMuteMask()
 {
+#if QT_VERSION <= QT_VERSION_CHECK(5,14,0)
     return QVector<bool>::fromStdVector(midiSeq->muteMask);
+#else
+    return QVector<bool>(midiSeq->muteMask.begin(), midiSeq->muteMask.end());
+#endif
 }
 
 void SeqWidget::handleController(int ccnumber, int channel, int value)
@@ -631,7 +691,7 @@ void SeqWidget::handleController(int ccnumber, int channel, int value)
             int sval = 0;
             bool m = false;
             switch (cclist.at(l2).ID) {
-                case 0: if (min == max) {
+                case MUTE_BUTTON: if (min == max) {
                             if (value == max) {
                                 m = midiSeq->isMuted;
                                 midiSeq->setMuted(!m);
@@ -647,17 +707,17 @@ void SeqWidget::handleController(int ccnumber, int channel, int value)
                         }
                 break;
 
-                case 1:
+                case SEQ_VELOCITY:
                         sval = min + ((double)value * (max - min) / 127);
                         midiSeq->updateVelocity(sval);
                 break;
 
-                case 2:
+                case SEQ_NOTE_LENGTH:
                         sval = min + ((double)value * (max - min) / 127);
                         midiSeq->updateNoteLength(sliderToTickLen(sval));
                 break;
 
-                case 3: if (min == max) {
+                case SEQ_RECORD: if (min == max) {
                             if (value == max) {
                                 m = midiSeq->recordMode;
                                 midiSeq->setRecordMode(!m);
@@ -673,19 +733,19 @@ void SeqWidget::handleController(int ccnumber, int channel, int value)
                             }
                         }
                 break;
-                case 4:
+                case SEQ_RESOLUTION:
                         sval = min + ((double)value * (max - min) / 127);
-                        if (sval < 5) resBoxIndex = sval;
+                        if ((uint64_t)sval < sizeof(seqResValues)/sizeof(seqResValues[0])) resBoxIndex = sval;
                 break;
-                case 5:
+                case SEQ_SIZE:
                         sval = min + ((double)value * (max - min) / 127);
-                        if (sval < 8) sizeBoxIndex = sval;
+                        if ((uint64_t)sval < sizeof(seqSizeValues)/sizeof(seqResValues[0])) sizeBoxIndex = sval;
                 break;
-                case 6:
+                case SEQ_LOOP_MODE:
                         sval = min + ((double)value * (max - min) / 127);
                         if (sval < 6) midiSeq->curLoopMode = sval;
                 break;
-                case 7:
+                case PARAM_RESTORE:
                         sval = min + ((double)value * (max - min) / 127);
                         if ((sval < parStore->list.count())
                                 && (sval != parStore->activeStore)
@@ -697,12 +757,12 @@ void SeqWidget::handleController(int ccnumber, int channel, int value)
                         else return;
                 break;
 
-                case 8:
+                case SEQ_TRANSPOSE:
                         sval = min + ((double)value * (max - min) / 127);
-                        midiSeq->updateTranspose(sval - 24);
+                        midiSeq->updateTranspose(sval - 36);
                 break;
                 
-                case 9:
+                case SEQ_CHANNEL_OUT:
                         sval = min + ((double)value * (max - min) / 127);
                         if (sval < 16) midiSeq->channelOut = sval;
                 break;
@@ -720,13 +780,26 @@ void SeqWidget::updateDisplay()
     QVector<Sample> data;
     std::vector<Sample> sdata;
 
-    parStore->updateDisplay(getFramePtr(), midiSeq->reverse);
+    bool repetitionsFinished = (midiSeq->currentRepetition == 0);
+    if (midiSeq->reverse) {
+        repetitionsFinished = (midiSeq->currentRepetition >= midiSeq->nRepetitions - 1);
+    }
+    parStore->updateDisplay(getFramePtr(), midiSeq->nPoints, repetitionsFinished, midiSeq->reverse);
+    if (parStore->nRepList.count() > 0) {
+        if (parStore->nRepList.at(parStore->activeStore) != midiSeq->nRepetitions) {
+            updateNRep(parStore->nRepList.at(parStore->activeStore));
+        }
+    }
 
     if (dataChanged || midiSeq->dataChanged) {
         dataChanged=false;
         midiSeq->dataChanged=false;
         midiSeq->getData(&sdata);
-        data = QVector<Sample>::fromStdVector(sdata);
+#if QT_VERSION <= QT_VERSION_CHECK(5,14,0)
+        data=QVector<Sample>::fromStdVector(sdata);
+#else
+        data=QVector<Sample>(sdata.begin(), sdata.end());
+#endif
         screen->updateData(data);
         if (recordMode) screen->setCurrentRecStep(midiSeq->currentRecStep);
         cursor->updateNumbers(midiSeq->res, midiSeq->size);
